def get_boolean_choice(prompt, default=True):
    if default:
        yn = 'Y/n'
    else:
        yn = 'y/N'

    choice = input(f'{prompt} [{yn}]: ')
    if not choice:
        return default
    elif choice[0].lower() == 'y':
        return True
    else:
        return False


def sdl_prompt(sdl_data, title):
    tags = ['']
    if '__required' in sdl_data:
        tags.extend(sdl_data['__required']['tags'])

    print(f'You are about to install {title}, this application supports selective downloads.')
    print('The following optional packs are available (tag - name):')
    for tag, info in sdl_data.items():
        if tag == '__required':
            continue
        print(' *', tag, '-', info['name'])

    examples = ', '.join([g for g in sdl_data.keys() if g != '__required'][:2])
    print(f'Please enter tags of pack(s) to install (space/comma-separated, e.g. "{examples}")')
    print('Leave blank to use defaults (only required data will be downloaded).')
    choices = input(f'Additional packs [Enter to confirm]: ')
    if not choices:
        return tags

    for c in choices.strip('"').replace(',', ' ').split():
        c = c.strip()
        if c in sdl_data:
            tags.extend(sdl_data[c]['tags'])
        else:
            print('Invalid tag:', c)

    return tags



import configparser
import logging
import os

logger = logging.getLogger('WineHelpers')


def read_registry(wine_pfx):
    reg = configparser.ConfigParser(comment_prefixes=(';', '#', '/', 'WINE'), allow_no_value=True)
    reg.optionxform = str
    reg.read(os.path.join(wine_pfx, 'user.reg'))
    return reg


def get_shell_folders(registry, wine_pfx):
    folders = dict()
    for k, v in registry['Software\\\\Microsoft\\\\Windows\\\\CurrentVersion\\\\Explorer\\\\Shell Folders'].items():
        path_cleaned = v.strip('"').strip().replace('\\\\', '/').replace('C:/', '')
        folders[k.strip('"').strip()] = os.path.join(wine_pfx, 'drive_c', path_cleaned)
    return folders


def case_insensitive_path_search(path):
    """
    Attempts to find a path case-insensitively
    """
    # Legendary's save path resolver always returns absolute paths, so this is not as horrible as it looks
    path_parts = path.replace('\\', '/').split('/')
    path_parts[0] = '/'
    # filter out empty parts
    path_parts = [i for i in path_parts if i]

    # attempt to find lowest level directory that exists case-sensitively
    longest_path = ''
    remaining_parts = []
    for i in range(len(path_parts), 0, -1):
        if os.path.exists(os.path.join(*path_parts[:i])):
            longest_path = path_parts[:i]
            remaining_parts = path_parts[i:]
            break
    logger.debug(f'Longest valid path: {longest_path}')
    logger.debug(f'Remaining parts: {remaining_parts}')

    # Iterate over remaining parts, find matching directories case-insensitively
    still_remaining = []
    for idx, part in enumerate(remaining_parts):
        for item in os.listdir(os.path.join(*longest_path)):
            if not os.path.isdir(os.path.join(*longest_path, item)):
                continue
            if item.lower() == part.lower():
                longest_path.append(item)
                break
        else:
            # once we stop finding parts break
            still_remaining = remaining_parts[idx-1:]
            break

    logger.debug(f'New longest path: {longest_path}')
    logger.debug(f'Still unresolved: {still_remaining}')
    final_path = os.path.join(*longest_path, *still_remaining)
    logger.debug('Final path:', final_path)
    return os.path.realpath(final_path)



# coding: utf-8

from asyncio import Queue as AIOQueue
from dataclasses import dataclass
from multiprocessing import Queue as MPQueue
from typing import Optional

from .manifest import Manifest
from .game import Game


@dataclass
class DownloaderManagerOptions:
    download_dir: str
    base_url: str
    cache_dir: Optional[str] = None
    resume_file: Optional[str] = None
    # queue options
    status_queue: Optional[MPQueue, AIOQueue] = None
    update_interval: float = 1.0
    # downloader options
    dl_timeout: float = 10.0
    max_workers: Optional[int] = None
    max_shared_memory: int = 1024 * 1024 * 1024


@dataclass
class DownloaderAnalysisOptions:
    manifest: Manifest
    old_manifest: Optional[Manifest] = None

    # additional features
    enable_patching: bool = True
    enable_resume: bool = True
    enable_process_opts: bool = False

    # filters
    prefix_include_filters: Optional[list] = None
    prefix_exclude_filters: Optional[list] = None
    suffix_include_filters: Optional[list] = None
    suffix_exclude_filters: Optional[list] = None
    install_tags: Optional[list] = None


@dataclass
class CorePrepareDownloadOpts:
    game: Game
    base_game: Game = None
    base_path: Optional[str] = None
    game_folder: Optional[str] = None

    # behavioural options
    force: bool = False
    repair: bool = False
    disable_delta: bool = False
    disable_https: bool = False
    repair_use_latest: bool = False

    # manifest options
    override_manifest: Optional[str] = None
    override_old_manifest: Optional[str] = None
    override_delta_manifest: Optional[str] = None
    override_base_url: Optional[str] = None
    override_platform: Optional[str] = None

    # downloader options
    preferred_cdn: Optional[str] = None
    disable_patching: bool = False
    disable_process_opts: bool = False
    status_queue: Optional[MPQueue, AIOQueue] = None
    max_shared_memory: Optional[int] = None
    max_workers: Optional[int] = None
    dl_timeout: Optional[float] = None
    # filtering
    prefix_include_filters: Optional[list] = None
    prefix_exclude_filters: Optional[list] = None
    suffix_include_filters: Optional[list] = None
    suffix_exclude_filters: Optional[list] = None


